@@ -1,11 +1,12 @@
 package Zonemaster::Engine::Profile;
 
-use version; our $VERSION = version->declare("v1.2.15");
-
 use 5.014002;
+
 use strict;
 use warnings;
 
+use version; our $VERSION = version->declare( "v1.2.22" );
+
 use File::ShareDir qw[dist_file];
 use JSON::PP qw( encode_json decode_json );
 use Scalar::Util qw(reftype);
@@ -15,7 +16,7 @@ use Data::Dumper;
 
 use Zonemaster::Engine;
 use Zonemaster::Engine::Net::IP;
-use Zonemaster::Engine::Constants qw( $RESOLVER_SOURCE_OS_DEFAULT );
+use Zonemaster::Engine::Constants qw( $RESOLVER_SOURCE_OS_DEFAULT $DURATION_12_HOURS_IN_SECONDS $DURATION_180_DAYS_IN_SECONDS );
 
 my %profile_properties_details = (
     q{resolver.defaults.debug} => {
@@ -72,15 +73,50 @@ my %profile_properties_details = (
     q{asnroots} => {
         type    => q{ArrayRef},
         test    => sub {
-                          foreach my $ndd ( @{$_[0]} ) {
-                              die "Property asnroots has a NULL item" if not defined $ndd;
-                              die "Property asnroots has a non scalar item" if not defined ref($ndd);
-                              die "Property asnroots has an item too long" if length($ndd) > 255;
-                              foreach my $label ( split /\./, $ndd ) {
-                                  die "Property asnroots has a non domain name item" if $label !~ /^[a-z0-9](?:[-a-z0-9]{0,61}[a-z0-9])?$/;
-                              }
-                          }
-                      }
+            foreach my $ndd ( @{$_[0]} ) {
+                die "Property asnroots has a NULL item" if not defined $ndd;
+                die "Property asnroots has a non scalar item" if not defined ref($ndd);
+                die "Property asnroots has an item too long" if length($ndd) > 255;
+                foreach my $label ( split /[.]/, $ndd ) {
+                    die "Property asnroots has a non domain name item" if $label !~ /^[a-z0-9](?:[-a-z0-9]{0,61}[a-z0-9])?$/;
+                }
+            }
+        }
+    },
+    q{asn_db.style} => {
+        type    => q{Str},
+        test    => sub {
+            if ( lc($_[0]) ne q{cymru} and lc($_[0]) ne q{ripe} ) {
+                die "Property asn_db.style has 2 possible values : Cymru or RIPE (case insensitive)";
+            }
+            $_[0] = lc($_[0]);
+        },
+        default => q{cymru}
+    },
+    q{asn_db.sources} => {
+        type    => q{HashRef},
+        test    => sub {
+            foreach my $db_style ( keys %{$_[0]} ) {
+                if ( lc($db_style) ne q{cymru} and lc($db_style) ne q{ripe} ) {
+                    die "Property asn_db.sources keys have 2 possible values : Cymru or RIPE (case insensitive)";
+                }
+                if ( not scalar @{ ${$_[0]}{$db_style} } ) {
+                    die "Property asn_db.sources.$db_style has no items";
+                }
+                else {
+                    foreach my $ndd ( @{ ${$_[0]}{$db_style} } ) {
+                        die "Property asn_db.sources.$db_style has a NULL item" if not defined $ndd;
+                        die "Property asn_db.sources.$db_style has a non scalar item" if not defined ref($ndd);
+                        die "Property asn_db.sources.$db_style has an item too long" if length($ndd) > 255;
+                        foreach my $label ( split /[.]/, $ndd ) {
+                            die "Property asn_db.sources.$db_style has a non domain name item" if $label !~ /^[a-z0-9](?:[-a-z0-9]{0,61}[a-z0-9])?$/;
+                        }
+                    }
+                    ${$_[0]}{lc($db_style)} = delete ${$_[0]}{$db_style};
+                }
+            }
+        },
+        default => { cymru => [ "asnlookup.zonemaster.net" ] }
     },
     q{logfilter} => {
         type    => q{HashRef},
@@ -91,6 +127,21 @@ my %profile_properties_details = (
     },
     q{test_cases} => {
         type    => q{ArrayRef}
+    },
+    q{test_cases_vars.dnssec04.REMAINING_SHORT} => {
+        type    => q{Num},
+        min     => 1,
+        default => $DURATION_12_HOURS_IN_SECONDS
+    },
+    q{test_cases_vars.dnssec04.REMAINING_LONG} => {
+        type    => q{Num},
+        min     => 1,
+        default => $DURATION_180_DAYS_IN_SECONDS
+    },
+    q{test_cases_vars.dnssec04.DURATION_LONG} => {
+        type    => q{Num},
+        min     => 1,
+        default => $DURATION_180_DAYS_IN_SECONDS
     }
 );
 
@@ -101,8 +152,8 @@ sub _init_profile_properties_details_defaults {
     my $json           = read_file( $default_file );
     my $default_values = decode_json( $json );
     foreach my $property_name ( keys %profile_properties_details ) {
-        if ( defined _get_value_from_nested_hash( $default_values, split /\./, $property_name ) ) {
-            $profile_properties_details{$property_name}{default} = clone _get_value_from_nested_hash( $default_values, split /\./, $property_name );
+        if ( defined _get_value_from_nested_hash( $default_values, split /[.]/, $property_name ) ) {
+            $profile_properties_details{$property_name}{default} = clone _get_value_from_nested_hash( $default_values, split /[.]/, $property_name );
         }
     }
 }
@@ -191,9 +242,9 @@ sub get {
     die "Unknown property '$property_name'"  if not exists $profile_properties_details{$property_name};
 
     if ( $profile_properties_details{$property_name}->{type} eq q{ArrayRef} or $profile_properties_details{$property_name}->{type} eq q{HashRef} ) {
-        return clone _get_value_from_nested_hash( $self->{q{profile}}, split /\./, $property_name );
+        return clone _get_value_from_nested_hash( $self->{q{profile}}, split /[.]/, $property_name );
     } else {
-        return _get_value_from_nested_hash( $self->{q{profile}}, split /\./, $property_name );
+        return _get_value_from_nested_hash( $self->{q{profile}}, split /[.]/, $property_name );
     }
 }
 
@@ -271,8 +322,8 @@ sub _set {
         $profile_properties_details{$property_name}->{test}->( $value );
     }
 
-    return _set_value_to_nested_hash( $self->{q{profile}}, $value, split /\./, $property_name );
-}   
+    return _set_value_to_nested_hash( $self->{q{profile}}, $value, split /[.]/, $property_name );
+}
 
 sub merge {
     my ( $self, $other_profile ) = @_;
@@ -280,8 +331,8 @@ sub merge {
     die "Merge with ", __PACKAGE__, " only" if ref($other_profile) ne __PACKAGE__;
 
     foreach my $property_name ( keys %profile_properties_details ) {
-        if ( defined _get_value_from_nested_hash( $other_profile->{q{profile}}, split /\./, $property_name ) ) {
-            $self->_set( q{JSON}, $property_name, _get_value_from_nested_hash( $other_profile->{q{profile}}, split /\./, $property_name ) );
+        if ( defined _get_value_from_nested_hash( $other_profile->{q{profile}}, split /[.]/, $property_name ) ) {
+            $self->_set( q{JSON}, $property_name, _get_value_from_nested_hash( $other_profile->{q{profile}}, split /[.]/, $property_name ) );
         }
     }
     return $other_profile->{q{profile}};
@@ -294,8 +345,8 @@ sub from_json {
     my %paths;
     _get_profile_paths(\%paths, $internal);
     foreach my $property_name ( keys %paths ) {
-        if ( defined _get_value_from_nested_hash( $internal, split /\./, $property_name ) ) {
-            $new->_set( q{JSON}, $property_name, _get_value_from_nested_hash( $internal, split /\./, $property_name ) );
+        if ( defined _get_value_from_nested_hash( $internal, split /[.]/, $property_name ) ) {
+            $new->_set( q{JSON}, $property_name, _get_value_from_nested_hash( $internal, split /[.]/, $property_name ) );
         }
     }
 
@@ -569,7 +620,7 @@ A boolean. If true, network traffic is forbidden. Default false.
 Use when you want to be sure that any data is only taken from a preloaded
 cache.
 
-=head2 asnroots
+=head2 asnroots (DEPRECATED)
 
 An arrayref of domain names. Default C<["asnlookup.zonemaster.net",
 "asnlookup.iis.se", "asn.cymru.com"]>.
@@ -578,6 +629,20 @@ The domains will be assumed to be Cymru-style AS lookup zones.
 Normally only the first name in the list will be used, the rest are
 backups in case the earlier ones don't work.
 
+=head2 asn_db.style
+
+A string that is either C<"Cymru"> or C<"RIPE">. Defines which method will
+be used for AS lookup zones.
+Default C<"Cymru">.
+
+=head2 asn_db.sources
+
+An arrayref of domain names when asn_db.style is set to C<"Cymru"> or whois
+servers when asn_db.style is set to C<"RIPE">. Normally only the first item
+in the list will be used, the rest are backups in case the earlier ones don't 
+work.
+Default C<"asnlookup.zonemaster.net">.
+
 =head2 logfilter
 
 A complex data structure. Default C<{}>.
@@ -669,7 +734,7 @@ modules emit.
 The values of the second level hashes are mapped to severity levels.
 
 The various L<test case specifications|
-https://github.com/zonemaster/zonemaster/tree/master/docs/specifications/tests>
+https://github.com/zonemaster/zonemaster/tree/master/docs/specifications/tests/README.md>
 define the default severity level for some of the messages.
 These specifications are the only authoritative documents on the default
 severity level for the various messages.
@@ -686,8 +751,9 @@ level for the tag.
 
 =head2 test_cases
 
-An arrayref of names of test cases as listed in the L<test case specifications|
-https://github.com/zonemaster/zonemaster/tree/master/docs/specifications/tests>.
+An arrayref of names of implemented test cases as listed in the
+L<test case specifications|
+https://github.com/zonemaster/zonemaster/tree/master/docs/specifications/tests/ImplementedTestCases.md>.
 Default is an arrayref listing all the test cases.
 
 Specifies which test cases to consider when a test module is asked 
@@ -700,6 +766,27 @@ matter if they're excluded from this property.
 This is because part of their function is to verify that the given domain name
 can be tested at all.
 
+=head2 test_cases_vars.dnssec04.REMAINING_SHORT
+
+A positive integer value.
+If the remaining validity time of the signature is less than test_cases_vars.dnssec04.REMAINING_SHORT (in seconds) 
+this test case returns the REMAINING_SHORT message tag.
+Default C<43200> (12 hours in seconds).
+
+=head2 test_cases_vars.dnssec04.REMAINING_LONG
+
+A positive integer value.
+If the remaining validity time of the signature is more than test_cases_vars.dnssec04.REMAINING_LONG (in seconds)
+this test case returns the REMAINING_LONG message tag.
+Default C<15552000> (180 days in seconds).
+
+=head2 test_cases_vars.dnssec04.DURATION_LONG
+
+A positive integer value.
+Returns DURATION_LONG message tag in case signature lifetime is more
+than test_cases_vars.dnssec04.DURATION_LONG (in seconds).
+Default C<15552000> (180 days in seconds).
+
 =head1 JSON REPRESENTATION
 
 Property names in L</PROFILE PROPERTIES> section correspond to paths in
