@@ -5,7 +5,15 @@ use 5.014002;
 use strict;
 use warnings;
 
-use version; our $VERSION = version->declare( "v1.1.10" );
+use version; our $VERSION = version->declare( "v1.1.12" );
+
+use Moose;
+
+use Readonly;
+use Module::Find;
+use Net::IP;
+use List::MoreUtils;
+use Clone;
 
 use Zonemaster::LDNS;
 use Zonemaster::Engine;
@@ -38,7 +46,6 @@ sub _log_versions {
     info( DEPENDENCY_VERSION => { name => 'IO::Socket::INET6',     version => $IO::Socket::INET6::VERSION } );
     info( DEPENDENCY_VERSION => { name => 'Moose',                 version => $Moose::VERSION } );
     info( DEPENDENCY_VERSION => { name => 'Module::Find',          version => $Module::Find::VERSION } );
-    info( DEPENDENCY_VERSION => { name => 'JSON',                  version => $JSON::VERSION } );
     info( DEPENDENCY_VERSION => { name => 'File::ShareDir',        version => $File::ShareDir::VERSION } );
     info( DEPENDENCY_VERSION => { name => 'File::Slurp',           version => $File::Slurp::VERSION } );
     info( DEPENDENCY_VERSION => { name => 'Net::IP',               version => $Net::IP::VERSION } );
@@ -79,11 +86,6 @@ sub run_all_for {
     if ( Zonemaster::Engine::Test::Basic->can_continue( @results ) and Zonemaster::Engine->can_continue() ) {
         foreach my $mod ( __PACKAGE__->modules ) {
 
-            if ( not _policy_allowed( $mod ) ) {
-                push @results, info( POLICY_DISABLED => { name => $mod } );
-                next;
-            }
-
             my $module = "Zonemaster::Engine::Test::$mod";
             info( MODULE_VERSION => { module => $module, version => $module->version } );
             my @res = eval { $module->all( $zone ) };
@@ -102,7 +104,7 @@ sub run_all_for {
         } ## end foreach my $mod ( __PACKAGE__...)
     } ## end if ( Zonemaster::Engine::Test::Basic...)
     else {
-        push @results, info( CANNOT_CONTINUE => { zone => $zone->name->string } );
+        push @results, info( CANNOT_CONTINUE => { domain => $zone->name->string } );
     }
 
     return @results;
@@ -140,11 +142,11 @@ sub run_module {
             return @res;
         }
         else {
-            info( UNKNOWN_MODULE => { name => $requested, method => 'all', known => join( ':', sort $class->modules ) } );
+            info( UNKNOWN_MODULE => { module => $requested, testcase => 'all', module_list => join( ':', sort $class->modules ) } );
         }
     }
     else {
-        info( CANNOT_CONTINUE => { zone => $zone->name->string } );
+        info( CANNOT_CONTINUE => { domain => $zone->name->string } );
     }
 
     return;
@@ -159,7 +161,7 @@ sub run_one {
     Zonemaster::Engine->start_time_now();
     push @res, info( START_TIME => { time_t => time(), string => strftime( "%F %T %z", ( localtime() ) ) } );
     push @res,
-      info( TEST_ARGS => { module => $requested, method => $test, args => join( ';', map { "$_" } @arguments ) } );
+      info( TEST_ARGS => { module => $requested, testcase => $test, args => join( ';', map { "$_" } @arguments ) } );
     _log_versions();
     if ( not( Zonemaster::Engine::Profile->effective->get( q{net.ipv4} ) or Zonemaster::Engine::Profile->effective->get( q{net.ipv6} ) ) ) {
         return info( NO_NETWORK => {} );
@@ -169,7 +171,7 @@ sub run_one {
         if ( $module ) {
             my $m = "Zonemaster::Engine::Test::$module";
             if ( $m->metadata->{$test} ) {
-                info( MODULE_CALL => { module => $module, method => $test, version => $m->version } );
+                info( MODULE_VERSION => { module => $m, version => $m->version } );
                 push @res, eval { $m->$test( @arguments ) };
                 if ( $@ ) {
                     my $err = $@;
@@ -180,15 +182,15 @@ sub run_one {
                         push @res, info( MODULE_ERROR => { module => $module, msg => "$err" } );
                     }
                 }
-                info( MODULE_CALL_END => { module => $module, method => $test } );
+                info( MODULE_END => { module => $module } );
                 return @res;
             }
             else {
-                info( UNKNOWN_METHOD => { module => $m, method => $test } );
+                info( UNKNOWN_METHOD => { module => $m, testcase => $test } );
             }
         } ## end if ( $module )
         else {
-            info( UNKNOWN_MODULE => { module => $requested, method => $test, known => join( ':', sort $class->modules ) } );
+            info( UNKNOWN_MODULE => { module => $requested, testcase => $test, module_list => join( ':', sort $class->modules ) } );
         }
     }
     else {
@@ -198,18 +200,12 @@ sub run_one {
                 $zname = $arg->name;
             }
         }
-        info( CANNOT_CONTINUE => { zone => $zname } );
+        info( CANNOT_CONTINUE => { domain => $zname } );
     }
 
     return;
 } ## end sub run_one
 
-sub _policy_allowed {
-    my ( $name ) = @_;
-
-    return not Zonemaster::Engine::Util::test_levels()->{ uc( $name ) }{DISABLED};
-}
-
 1;
 
 =head1 NAME
