@@ -1,11 +1,11 @@
 package Zonemaster::Engine::Test::Basic;
 
-use version; our $VERSION = version->declare("v1.0.12");
+use 5.014002;
 
 use strict;
 use warnings;
 
-use 5.014002;
+use version; our $VERSION = version->declare("v1.0.18");
 
 use Zonemaster::Engine;
 
@@ -13,6 +13,7 @@ use Carp;
 use List::MoreUtils qw[any none];
 use Locale::TextDomain qw[Zonemaster-Engine];
 use Readonly;
+use Zonemaster::Engine::Profile;
 use Zonemaster::Engine::Constants qw[:ip :name];
 use Zonemaster::Engine::Test::Address;
 use Zonemaster::Engine::Test::Syntax;
@@ -40,6 +41,8 @@ sub all {
     {
         push @results, $class->basic01( $zone );
 
+        push @results, $class->basic04( $zone );
+
         push @results, $class->basic02( $zone );
 
         # Perform BASIC3 if BASIC2 failed
@@ -54,6 +57,7 @@ sub all {
                 }
               );
         }
+
     } ## end if ( none { $_->tag eq...})
 
     return @results;
@@ -84,12 +88,16 @@ sub metadata {
               DOMAIN_NAME_LABEL_TOO_LONG
               DOMAIN_NAME_ZERO_LENGTH_LABEL
               DOMAIN_NAME_TOO_LONG
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         basic01 => [
             qw(
               NO_PARENT
               HAS_PARENT
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         basic02 => [
@@ -102,90 +110,172 @@ sub metadata {
               IPV6_DISABLED
               IPV4_ENABLED
               IPV6_ENABLED
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
         basic03 => [
             qw(
-              NO_A_RECORDS
+              A_QUERY_NO_RESPONSES
               HAS_A_RECORDS
               IPV4_DISABLED
-              IPV6_DISABLED
               IPV4_ENABLED
+              IPV6_DISABLED
               IPV6_ENABLED
-              A_QUERY_NO_RESPONSES
+              NO_A_RECORDS
+              TEST_CASE_END
+              TEST_CASE_START
+              )
+        ],
+        basic04 => [
+            qw(
+              B04_MISSING_NS_RECORD
+              B04_MISSING_SOA_RECORD
+              B04_NO_RESPONSE
+              B04_NO_RESPONSE_NS_QUERY
+              B04_NO_RESPONSE_SOA_QUERY
+              B04_NS_RECORD_NOT_AA
+              B04_RESPONSE_TCP_NOT_UDP
+              B04_SOA_RECORD_NOT_AA
+              B04_UNEXPECTED_RCODE_NS_QUERY
+              B04_UNEXPECTED_RCODE_SOA_QUERY
+              B04_WRONG_NS_RECORD
+              B04_WRONG_SOA_RECORD
+              IPV4_DISABLED
+              IPV4_ENABLED
+              IPV6_DISABLED
+              IPV6_ENABLED
+              TEST_CASE_END
+              TEST_CASE_START
               )
         ],
     };
 } ## end sub metadata
 
 Readonly my %TAG_DESCRIPTIONS => (
+    A_QUERY_NO_RESPONSES => sub {
+        __x    # BASIC:A_QUERY_NO_RESPONSES
+          'Nameservers did not respond to A query.';
+    },
+    B04_MISSING_NS_RECORD => sub {
+        __x    # BASIC:B04_MISSING_NS_RECORD
+          'Nameserver {ns} reponds to a NS query with no NS records in the answer section.', @_;
+    },
+    B04_MISSING_SOA_RECORD => sub {
+        __x    # BASIC:B04_MISSING_SOA_RECORD
+          'Nameserver {ns} reponds to a SOA query with no SOA records in the answer section.', @_;
+    },
+    B04_NO_RESPONSE => sub {
+        __x    # BASIC:B04_NO_RESPONSE
+          'Nameserver {ns} does not respond over neither UDP nor TCP.', @_;
+    },
+    B04_NO_RESPONSE_NS_QUERY => sub {
+        __x    # BASIC:B04_NO_RESPONSE_NS_QUERY
+          'Nameserver {ns} does not respond to NS queries.', @_;
+    },
+    B04_NO_RESPONSE_SOA_QUERY => sub {
+        __x    # BASIC:B04_NO_RESPONSE_SOA_QUERY
+          'Nameserver {ns} does not respond to SOA queries.', @_;
+    },
+    B04_NS_RECORD_NOT_AA => sub {
+        __x    # BASIC:B04_NS_RECORD_NOT_AA
+          'Nameserver {ns} does not give an authoritative response on an NS query.', @_;
+    },
+    B04_RESPONSE_TCP_NOT_UDP => sub {
+        __x    # BASIC:B04_RESPONSE_TCP_NOT_UDP
+          'Nameserver {ns} does not respond over UDP.', @_;
+    },
+    B04_SOA_RECORD_NOT_AA => sub {
+        __x    # BASIC:B04_SOA_RECORD_NOT_AA
+          'Nameserver {ns} does not give an authoritative response on an SOA query.', @_;
+    },
+    B04_UNEXPECTED_RCODE_NS_QUERY => sub {
+        __x    # BASIC:B04_UNEXPECTED_RCODE_NS_QUERY
+          'Nameserver {ns} responds with an unexpected RCODE ({rcode}) on an NS query.', @_;
+    },
+    B04_UNEXPECTED_RCODE_SOA_QUERY => sub {
+        __x    # BASIC:B04_UNEXPECTED_RCODE_SOA_QUERY
+          'Nameserver {ns} responds with an unexpected RCODE ({rcode}) on an SOA query.', @_;
+    },
+    B04_WRONG_NS_RECORD => sub {
+        __x    # BASIC:B04_WRONG_NS_RECORD
+          'Nameserver {ns} responds with a wrong owner name ({owner} instead of {name}) on NS queries.', @_;
+    },
+    B04_WRONG_SOA_RECORD => sub {
+        __x    # BASIC:B04_WRONG_SOA_RECORD
+          'Nameserver {ns} responds with a wrong owner name ({owner} instead of {name}) on SOA queries.', @_;
+    },
     DOMAIN_NAME_LABEL_TOO_LONG => sub {
         __x    # BASIC:DOMAIN_NAME_LABEL_TOO_LONG
-          "Domain name ({dname}) has a label ({dlabel}) too long ({dlength}/{max}).", @_;
-    },
-    DOMAIN_NAME_ZERO_LENGTH_LABEL => sub {
-        __x    # BASIC:DOMAIN_NAME_ZERO_LENGTH_LABEL
-          "Domain name ({dname}) has a zero length label.", @_;
+          'Domain name ({domain}) has a label ({dlabel}) too long ({dlength}/{max}).', @_;
     },
     DOMAIN_NAME_TOO_LONG => sub {
         __x    # BASIC:DOMAIN_NAME_TOO_LONG
-          "Domain name is too long ({fqdnlength}/{max}).", @_;
+          'Domain name is too long ({fqdnlength}/{max}).', @_;
     },
-    NO_PARENT => sub {
-        __x    # BASIC:NO_PARENT
-          'No parent domain could be found for the tested domain.', @_;
+    DOMAIN_NAME_ZERO_LENGTH_LABEL => sub {
+        __x    # BASIC:DOMAIN_NAME_ZERO_LENGTH_LABEL
+          'Domain name ({domain}) has a zero-length label.', @_;
+    },
+    HAS_A_RECORDS => sub {
+        __x    # BASIC:HAS_A_RECORDS
+          'Nameserver {ns} returned "A" record(s) for {domain}.', @_;
+    },
+    HAS_NAMESERVER_NO_WWW_A_TEST => sub {
+        __x    # BASIC:HAS_NAMESERVER_NO_WWW_A_TEST
+          'Functional nameserver found. "A" query for www.{zname} test skipped.', @_;
+    },
+    HAS_NAMESERVERS => sub {
+        __x    # BASIC:HAS_NAMESERVERS
+          'Nameserver {ns} listed these servers as glue: {nsnlist}.', @_;
     },
     HAS_PARENT => sub {
         __x    # BASIC:HAS_PARENT
           'Parent domain \'{pname}\' was found for the tested domain.', @_;
     },
-    HAS_A_RECORDS => sub {
-        __x    # BASIC:HAS_A_RECORDS
-          'Nameserver {ns}/{address} returned A record(s) for {dname}.', @_;
-    },
-    NO_A_RECORDS => sub {
-        __x    # BASIC:NO_A_RECORDS
-          'Nameserver {ns}/{address} did not return A record(s) for {dname}.', @_;
-    },
-    HAS_NAMESERVERS => sub {
-        __x    # BASIC:HAS_NAMESERVERS
-          'Nameserver {ns}/{address} listed these servers as glue: {nsnlist}.', @_;
-    },
-    NO_GLUE_PREVENTS_NAMESERVER_TESTS => sub {
-        __x    # BASIC:NO_GLUE_PREVENTS_NAMESERVER_TESTS
-          'No NS records for tested zone from parent. NS tests aborted.', @_;
-    },
-    NS_FAILED => sub {
-        __x    # BASIC:NS_FAILED
-          'Nameserver {ns}/{address} did not return NS records. RCODE was {rcode}.', @_;
-    },
-    NS_NO_RESPONSE => sub {
-        __x    # BASIC:NS_NO_RESPONSE
-          'Nameserver {ns}/{address} did not respond to NS query.', @_;
-    },
-    A_QUERY_NO_RESPONSES => sub {
-        __x    # BASIC:A_QUERY_NO_RESPONSES
-          'Nameservers did not respond to A query.';
-    },
-    HAS_NAMESERVER_NO_WWW_A_TEST => sub {
-        __x    # BASIC:HAS_NAMESERVER_NO_WWW_A_TEST
-          'Functional nameserver found. "A" query for www.{zname} test aborted.', @_;
-    },
     IPV4_DISABLED => sub {
         __x    # BASIC:IPV4_DISABLED
-          'IPv4 is disabled, not sending "{rrtype}" query to {ns}/{address}.', @_;
+          'IPv4 is disabled, not sending "{rrtype}" query to {ns}.', @_;
     },
     IPV4_ENABLED => sub {
         __x    # BASIC:IPV4_ENABLED
-          'IPv4 is enabled, can send "{rrtype}" query to {ns}/{address}.', @_;
+          'IPv4 is enabled, can send "{rrtype}" query to {ns}.', @_;
     },
     IPV6_DISABLED => sub {
         __x    # BASIC:IPV6_DISABLED
-          'IPv6 is disabled, not sending "{rrtype}" query to {ns}/{address}.', @_;
+          'IPv6 is disabled, not sending "{rrtype}" query to {ns}.', @_;
     },
     IPV6_ENABLED => sub {
         __x    # BASIC:IPV6_ENABLED
-          'IPv6 is enabled, can send "{rrtype}" query to {ns}/{address}.', @_;
+          'IPv6 is enabled, can send "{rrtype}" query to {ns}.', @_;
+    },
+    NO_A_RECORDS => sub {
+        __x    # BASIC:NO_A_RECORDS
+          'Nameserver {ns} did not return "A" record(s) for {domain}.', @_;
+    },
+    NO_GLUE_PREVENTS_NAMESERVER_TESTS => sub {
+        __x    # BASIC:NO_GLUE_PREVENTS_NAMESERVER_TESTS
+          'No NS records for tested zone from parent. NS tests skipped.', @_;
+    },
+    NO_PARENT => sub {
+        __x    # BASIC:NO_PARENT
+          'No parent domain could be found for the domain under test.', @_;
+    },
+    NS_FAILED => sub {
+        __x    # BASIC:NS_FAILED
+          'Nameserver {ns} did not return NS records. RCODE was {rcode}.', @_;
+    },
+    NS_NO_RESPONSE => sub {
+        __x    # BASIC:NS_NO_RESPONSE
+          'Nameserver {ns} did not respond to NS query.', @_;
+    },
+    TEST_CASE_END => sub {
+        __x    # BASIC:TEST_CASE_END
+          'TEST_CASE_END {testcase}.', @_;
+    },
+    TEST_CASE_START => sub {
+        __x    # BASIC:TEST_CASE_START
+          'TEST_CASE_START {testcase}.', @_;
     },
 );
 
@@ -203,15 +293,15 @@ sub version {
 
 sub basic00 {
     my ( $class, $zone ) = @_;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
     my $name = name( $zone );
-    my @results;
 
     foreach my $local_label ( @{ $name->labels } ) {
         if ( length $local_label > $LABEL_MAX_LENGTH ) {
             push @results,
               info(
                 q{DOMAIN_NAME_LABEL_TOO_LONG} => {
-                    dname   => "$name",
+                    domain  => "$name",
                     dlabel  => $local_label,
                     dlength => length( $local_label ),
                     max     => $LABEL_MAX_LENGTH,
@@ -222,7 +312,7 @@ sub basic00 {
             push @results,
               info(
                 q{DOMAIN_NAME_ZERO_LENGTH_LABEL} => {
-                    dname => "$name",
+                    domain => "$name",
                 }
               );
         }
@@ -240,13 +330,13 @@ sub basic00 {
           );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 
 } ## end sub basic00
 
 sub basic01 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
     my $parent = $zone->parent;
 
     if ( not $parent ) {
@@ -267,12 +357,12 @@ sub basic01 {
           );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub basic01
 
 sub basic02 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
     my $query_type = q{NS};
     my @ns = @{ Zonemaster::Engine::TestMethods->method4( $zone ) };
 
@@ -288,9 +378,8 @@ sub basic02 {
             push @results,
               info(
                 IPV4_DISABLED => {
-                    ns      => $ns->name->string,
-                    address => $ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -299,9 +388,8 @@ sub basic02 {
             push @results,
               info(
                 IPV4_ENABLED => {
-                    ns      => $ns->name->string,
-                    address => $ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $ns->string,
+                    rrtype => $query_type,
                 }
               );
         }
@@ -310,9 +398,8 @@ sub basic02 {
             push @results,
               info(
                 IPV6_DISABLED => {
-                    ns      => $ns->name->string,
-                    address => $ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -321,9 +408,8 @@ sub basic02 {
             push @results,
               info(
                 IPV6_ENABLED => {
-                    ns      => $ns->name->string,
-                    address => $ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $ns->string,
+                    rrtype => $query_type,
                 }
               );
         }
@@ -337,8 +423,7 @@ sub basic02 {
                     HAS_NAMESERVERS => {
                         nsnlist =>
                           join( q{,}, sort map { $_->nsdname } $p->get_records_for_name( $query_type, $zone->name ) ),
-                        ns      => $ns->name->string,
-                        address => $ns->address->short,
+                        ns => $ns->string,
                     }
                   );
             }
@@ -346,30 +431,23 @@ sub basic02 {
                 push @results,
                   info(
                     NS_FAILED => {
-                        ns      => $ns->name->string,
-                        address => $ns->address->short,
-                        rcode   => $p->rcode,
+                        ns    => $ns->string,
+                        rcode => $p->rcode,
                     }
                   );
             }
         } ## end if ( $p )
         else {
-            push @results,
-              info(
-                NS_NO_RESPONSE => {
-                    ns      => $ns->name->string,
-                    address => $ns->address->short,
-                }
-              );
+            push @results, info( NS_NO_RESPONSE => { ns => $ns->string } );
         }
     } ## end foreach my $ns ( @{ Zonemaster::Engine::TestMethods...})
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub basic02
 
 sub basic03 {
     my ( $class, $zone ) = @_;
-    my @results;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
     my $query_type = q{A};
 
     my $name        = q{www.} . $zone->name;
@@ -379,9 +457,8 @@ sub basic03 {
             push @results,
               info(
                 IPV4_DISABLED => {
-                    ns      => $ns->name->string,
-                    address => $ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -390,9 +467,8 @@ sub basic03 {
             push @results,
               info(
                 IPV4_ENABLED => {
-                    ns      => $ns->name->string,
-                    address => $ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $ns->string,
+                    rrtype => $query_type,
                 }
               );
         }
@@ -401,9 +477,8 @@ sub basic03 {
             push @results,
               info(
                 IPV6_DISABLED => {
-                    ns      => $ns->name->string,
-                    address => $ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $ns->string,
+                    rrtype => $query_type,
                 }
               );
             next;
@@ -412,9 +487,8 @@ sub basic03 {
             push @results,
               info(
                 IPV6_ENABLED => {
-                    ns      => $ns->name->string,
-                    address => $ns->address->short,
-                    rrtype  => $query_type,
+                    ns     => $ns->string,
+                    rrtype => $query_type,
                 }
               );
         }
@@ -426,9 +500,8 @@ sub basic03 {
             push @results,
               info(
                 HAS_A_RECORDS => {
-                    ns      => $ns->name->string,
-                    address => $ns->address->short,
-                    dname   => $name,
+                    ns     => $ns->string,
+                    domain => $name,
                 }
               );
         }
@@ -436,9 +509,8 @@ sub basic03 {
             push @results,
               info(
                 NO_A_RECORDS => {
-                    ns      => $ns->name->string,
-                    address => $ns->address->short,
-                    dname   => $name,
+                    ns     => $ns->string,
+                    domain => $name,
                 }
               );
         }
@@ -448,9 +520,191 @@ sub basic03 {
         push @results, info( A_QUERY_NO_RESPONSES => {} );
     }
 
-    return @results;
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
 } ## end sub basic03
 
+sub basic04 {
+    my ( $class, $zone ) = @_;
+    push my @results, info( TEST_CASE_START => { testcase => (split /::/, (caller(0))[3])[-1] } );
+    my $name = name( $zone );
+    my @query_types = qw{SOA NS};
+    my @ns = @{ Zonemaster::Engine::TestMethods->method4and5( $zone ) };
+
+    foreach my $ns ( @ns ) {
+        if ( not Zonemaster::Engine::Profile->effective->get(q{net.ipv4}) and $ns->address->version == $IP_VERSION_4 ) {
+            push @results, map {
+              info(
+                IPV4_DISABLED => {
+                    ns     => $ns->string,
+                    rrtype => $_,
+                }
+              )
+            } @query_types;
+            next;
+        }
+        elsif ( Zonemaster::Engine::Profile->effective->get(q{net.ipv4}) and $ns->address->version == $IP_VERSION_4 ) {
+            push @results, map {
+              info(
+                IPV4_ENABLED => {
+                    ns     => $ns->string,
+                    rrtype => $_,
+                }
+              )
+            } @query_types;
+        }
+
+
+        if ( not Zonemaster::Engine::Profile->effective->get(q{net.ipv6}) and $ns->address->version == $IP_VERSION_6 ) {
+            push @results, map {
+              info(
+                IPV6_DISABLED => {
+                    ns     => $ns->string,
+                    rrtype => $_,
+                }
+              )
+            } @query_types;
+            next;
+        }
+        elsif ( Zonemaster::Engine::Profile->effective->get(q{net.ipv6}) and $ns->address->version == $IP_VERSION_6 ) {
+            push @results, map {
+              info(
+                IPV6_ENABLED => {
+                    ns     => $ns->string,
+                    rrtype => $_,
+                }
+              )
+            } @query_types;
+        }
+
+        my $p_soa_udp = $ns->query( $name, q{SOA}, { usevc => 0 } );
+        my $p_ns_udp  = $ns->query( $name, q{NS}, { usevc => 0 } );
+
+        if ( not $p_soa_udp and not $p_ns_udp ) {
+            my $p_soa_tcp = $ns->query( $name, q{SOA}, { usevc => 1 } );
+            if ( not $p_soa_tcp ) {
+                push @results,
+                  info(
+                    B04_NO_RESPONSE => {
+                        ns => $ns->string
+                    }
+                  );
+            }
+            else {
+                push @results,
+                  info(
+                    B04_RESPONSE_TCP_NOT_UDP => {
+                        ns => $ns->string
+                    }
+                  );
+            }
+        }
+        else {
+            if ( not $p_soa_udp ) {
+                push @results,
+                  info(
+                    B04_NO_RESPONSE_SOA_QUERY => {
+                        ns => $ns->string
+                    }
+                  );
+            }
+            else {
+                if ( $p_soa_udp->rcode ne q{NOERROR} ) {
+                    push @results,
+                      info(
+                        B04_UNEXPECTED_RCODE_SOA_QUERY => {
+                            ns => $ns->string,
+                            rcode => $p_soa_udp->rcode
+                        }
+                      );
+                }
+                else {
+                    my ( $soa ) = $p_soa_udp->get_records( q{SOA}, q{answer} );
+                    if ( not $soa ) {
+                        push @results,
+                          info(
+                            B04_MISSING_SOA_RECORD => {
+                                ns => $ns->string
+                            }
+                          );
+                    }
+                    else {
+                        if ( lc($soa->owner) ne lc($name->fqdn) ) {
+                            push @results,
+                              info(
+                                B04_WRONG_SOA_RECORD => {
+                                    ns    => $ns->string,
+                                    owner => lc($soa->owner),
+                                    name  => lc($name->fqdn)
+                                }
+                              );
+                        }
+                        elsif ( not $p_soa_udp->aa ) {
+                            push @results,
+                              info(
+                                B04_SOA_RECORD_NOT_AA => {
+                                    ns => $ns->string
+                                }
+                              );
+                        }
+                    }
+                }
+            }
+            if ( not $p_ns_udp ) {
+                push @results,
+                  info(
+                    B04_NO_RESPONSE_NS_QUERY => {
+                        ns => $ns->string
+                    }
+                  );
+            }
+            else {
+                if ( $p_ns_udp->rcode ne q{NOERROR} ) {
+                    push @results,
+                      info(
+                        B04_UNEXPECTED_RCODE_NS_QUERY => {
+                            ns    => $ns->string,
+                            rcode => $p_ns_udp->rcode
+                        }
+                      );
+                }
+                else {
+                    my ( $ns_in_answer ) = $p_ns_udp->get_records( q{NS}, q{answer} );
+                    if ( not $ns_in_answer ) {
+                        push @results,
+                          info(
+                            B04_MISSING_NS_RECORD => {
+                                ns => $ns->string
+                            }
+                          );
+                    }
+                    else {
+                        if ( lc($ns_in_answer->owner) ne lc($name->fqdn) ) {
+                            push @results,
+                              info(
+                                B04_WRONG_NS_RECORD => {
+                                    ns    => $ns->string,
+                                    owner => lc($ns_in_answer->owner),
+                                    name  => lc($name->fqdn)
+                                }
+                              );
+                        }
+                        elsif ( not $p_ns_udp->aa ) {
+                            push @results,
+                              info(
+                                B04_NS_RECORD_NOT_AA => {
+                                    ns => $ns->string
+                                }
+                              );
+                        }
+                    }
+                }
+            }
+        }
+    }
+
+    return ( @results, info( TEST_CASE_END => { testcase => (split /::/, (caller(0))[3])[-1] } ) );
+} ## end sub basic04
+
 1;
 
 =head1 NAME
@@ -512,6 +766,11 @@ Checks if at least one of the nameservers pointed out by the parent zone gives a
 tested zone (that is, if we're testing C<example.org> this test will as for A records for C<www.example.org>). This test is only run if the
 L<basic02> test has I<failed>.
 
+=item basic04
+
+Query all nameservers pointed out by the parent zone or found in delegation for NS and/or SOA records. Previously done in several
+test cases, these tests should be done only here.
+
 =back
 
 =cut
